%
% Example 1 of the paper "On the Dynamics of Hopfield Neural Networks on Unit Quaternions"
% accepted to be publieshed by IEEE TNNLS.
% 

pkg load quaternion; 
graphics_toolkit('gnuplot'); 
more off;
clear;
clc;

W=quaternion([0 5; 5 0],[0 1; -1 0],[0 7; -7 0],[0 2; -2 0]);
x=AngInv(-pi*[1/2;1/2],-pi*[1/8;1/8],-pi*[1/4;1/4]);
n=length(x);

K=[2 2 2];
tmax=10;

disp('Asynchronous MV-QHNN');
[xA,EnergyA,itA]=MvQHNN1(W,x,K,tmax);

disp('Parallel MV-QHNN');
[xP,EnergyP,itP]=ParallelMvQHNN1(W,x,K,tmax);

figure(1); clf;
H=plot([0:length(EnergyA)-1]/(3*n),EnergyA,'-ob','markerfacecolor','b',...
      [0:length(EnergyP)-1]/3,EnergyP,'-sr','markerfacecolor','r');
grid;
xlabel('t');
ylabel('Energy');
legend('Asynchronous','Parallel','location','northwest','orientation','horizontal');
set(H,'linewidth',6,'Markersize',8);
set(gca,'xtick',[0:1:tmax]);
print(sprintf('Example1TNNLS.eps',n,K(1,1),K(1,2),K(1,3)),'-depsc','-F:18');