%
% Example 4 of the paper "On the Dynamics of Hopfield Neural Networks on Unit Quaternions"
% accepted to be publieshed by IEEE TNNLS.
% 

graphics_toolkit('gnuplot'); 
more off;
clear;
clc;

for lambda=1:20
  K(lambda,1)=2^lambda;
  K(lambda,2)=2^lambda;
  K(lambda,3)=2^lambda;
end

n=100;
tmax=1000;
trial=100;

CONVERG=zeros(size(K,1),trial,6);

for ki=1:size(K,1)
  Kvalues{ki}=sprintf('[%d,%d,%d]',K(ki,1),K(ki,2),K(ki,3));
  
  delta_phi = 2*pi/K(ki,1);
  delta_psi = pi/(2*K(ki,2));
  delta_theta = pi/K(ki,3);  
  
  for t=1:trial
    disp('-------------------');
    disp(sprintf('Simulação:%d. K=[%d,%d,%d]',trial-t,K(ki,1),K(ki,2),K(ki,3)));
    
    % Generate W and x;
    x=quaternion(zeros(n,1));
    W=quaternion(zeros(n,n));
    
    f_1 = randi(K(ki,1),n,1)-1;
    f_2 = randi(K(ki,2),n,1)-1;
    f_3 = randi(K(ki,3),n,1)-1;
    phi=0.5*(-2*pi+delta_phi*(2*f_1+1));
    psi=0.5*(-pi/2+delta_psi*(2*f_2+1));
    theta=0.5*(-pi+delta_theta*(2*f_3+1));
    x=AngInv(phi,psi,theta);
    W=quaternion(2*rand(n)-1,2*rand(n)-1,2*rand(n)-1,2*rand(n)-1);
    W=(W+W')/2;
    W=W-diag(diag(W));

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('MV-QHNN1');
    [x1A,Energy1A,CONVERG(ki,t,1)]=MvQHNN1(W,x,K(ki,:),tmax);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('Parallel MV-QHNN1');
    [x1P,Energy1P,CONVERG(ki,t,2)]=ParallelMvQHNN1(W,x,K(ki,:),tmax);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('MV-QHNN3');
    [x3A,Energy3A,CONVERG(ki,t,3)]=MvQHNN3(W,x,K(ki,:),tmax);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('Parallel MV-QHNN3');
    [x3P,Energy3P,CONVERG(ki,t,4)]=ParallelMvQHNN3(W,x,K(ki,:),tmax);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('CV-QHNN');
    [xcA,EnergycA,CONVERG(ki,t,5)]=CvQHNN(W,x,tmax);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    disp('Parallel CV-QHNN');
    [xcP,EnergycP,CONVERG(ki,t,6)]=ParallelCvQHNN(W,x,tmax);    
  end
  
  figure(1);
  ind=[1:20];
  H=plot(ind,mean((CONVERG(ind,:,1)<tmax),2),'-or','markerfacecolor','r',...
    ind,mean((CONVERG(ind,:,3)<tmax),2),'-sb','markerfacecolor','b',...
    ind,mean((CONVERG(ind,:,5)<tmax),2),'-^g','markerfacecolor','g',...
    ind,mean((CONVERG(ind,:,2)<tmax),2),':or',...
    ind,mean((CONVERG(ind,:,4)<tmax),2),':sb',...
    ind,mean((CONVERG(ind,:,6)<tmax),2),':^g');
  grid; 
  legend('Asynchronous MV-QHNN','Asynchronous MV-QHNN3','Asynchronous CV-QHNN','Parallel MV-QHNN','Parallel MV-QHNN3','Parallel CV-QHNN','location','northwestoutside','orientation','horizontal');
  xlabel('K_1,K_2,K_3'); ylabel('Probability');
  axis([min(ind) max(ind) 0 1]);
  set(H,'linewidth',6,'Markersize',8);

  xticks = ind;
  set(gca, 'XTick', xticks);
  for j = 1:length(xticks)
    xtl{j} = sprintf('2^{%d}',xticks(j));
  end
  set(gca, 'XTickLabel', xtl)
  print(sprintf('ExampleProbabilities.eps',n),'-depsc','-F:15');
  
  save('-mat',sprintf('Example4_N%d.mat',n),'n','CONVERG','K');
end
