function [y,Energy,it]=MvQHNN3(W,x,K,it_max);

%
% -- Function File: y = MvQHNN3(W,x)
% -- Function File: y = MvQHNN3(W,x,K)
% -- Function File: y = MvQHNN3(W,x,K,it_max)
% -- Function File: [y,Energy,it] = MvQHNN3(W,x,K,it_max)
%
% Evaluates the modified version of the multivalued-valued quaternionic Hopfield neural network of Minemoto et al. using determinist 
%   asynchronous update. 
% Here, W is the quaternionic synaptic weight matrix and x is the input vector, where each component x_i is a
%   unit quaternion. The optional inputs K=[K1, K2, K3] and it_max asserts the resoluction factors and the maximum number of iteration 
%   (default are K=[256 256 256] and it_max = 100). 
% The output "Energy" contains the energy computed each time a neuron is updated. The output it yields the 
%   number of iteration. Here, all neurons are updated in one iteration.
%
% Warning: This code is designed for GNU Octave. It does not work on MATLAB!
%
% For more information, see the paper "On the Dynamics of Hopfield Neural Networks on Unit Quaternions"
% accepted to be publieshed by IEEE TNNLS.
%

if exist('mexMvQHNN3')~=3
  mkoctfile('mexMvQHNN3.c','--mex')
end

if nargin<3
  K=256*[1 1 1];
end

if nargin<4
  it_max=100;
end

[y_w,y_x,y_y,y_z,Energy,it]=mexMvQHNN3(W.w,W.x,W.y,W.z,x.w,x.x,x.y,x.z,K(1),K(2),K(3),it_max);
Energy=Energy([1:it*length(x)+1]);
y=quaternion(y_w,y_x,y_y,y_z);
