function [y,Energy,it]=ParallelCvQHNN(W,x,it_max);

%
% -- Function File: y = ParallelCvQHNN(W,x)
% -- Function File: y = ParallelCvQHNN(W,x,it_max)
% -- Function File: [y,Energy,it] = ParallelCvQHNN(W,x,it_max)
%
% Evaluates the continuous-valued quaternionic Hopfield neural network using parallel update. 
% Here, W is the quaternionic synaptic weight matrix and x is the input vector, where each component x_i is an
%   unit quaternion. The optional input it_max asserts a maximum number of iteration (default it_max = 100). 
% The output "Energy" contains the energy computed each time a neuron is updated. The output it yields the 
%   number of iteration. Here, all neurons are updated in one iteration.
%
% Warning: This code is designed for GNU Octave. It does not work on MATLAB!
%
% For more information, see the paper "On the Dynamics of Hopfield Neural Networks on Unit Quaternions"
% accepted to be publieshed by IEEE TNNLS.
%

if exist('mexParallelCvQHNN')~=3
  mkoctfile('mexParallelCvQHNN.c','--mex')
end

if nargin<3
  it_max=100;
end

[y_w,y_x,y_y,y_z,Energy,it]=mexParallelCvQHNN(W.w,W.x,W.y,W.z,x.w,x.x,x.y,x.z,it_max);
Energy=Energy([1:it+1]);
y=quaternion(y_w,y_x,y_y,y_z);
